@@ -8,8 +8,8 @@ import pytest
 from swh.scheduler.tests.conftest import *  # noqa
 
 
-@pytest.fixture(scope='session')  # type: ignore  # expected redefinition
+@pytest.fixture(scope="session")  # type: ignore  # expected redefinition
 def celery_includes():
     return [
-        'swh.loader.git.tasks',
+        "swh.loader.git.tasks",
     ]
